# -*- coding: utf-8 -*-
u"""Расширение поведения экшнов."""
from m3_ext.ui.results import ExtUIScriptResult
from .ui import BaseEditWinExtender


class BaseEditWinListener(object):

    u"""
    Базовый листенер для расширения окна редактирования доп.полями и данными.

    Окно редактирования связано с определенной моделью, она - расширяемая и ее
    данные требуется дополнить с помощью доп.полей.
    """

    # класс-наследник BaseEditWinExtender
    # отвечает за расширение интерфейса окна и биндинг
    ui_extender_cls = None
    # имя поля, которое ссылается на расширяемую модель
    parent_model_field = None

    def _get_id(self, context):
        u"""
        Получение из контекста параметра, определяющего id расширяемой модели.

        :rype: int
        """
        raise NotImplementedError()

    def _get_instance(self, row_id):
        u"""
        Получение модели, которая расширяет основную ``ui_extender_cls.model``.

        :param row_id: идентификатор расширяемой модели
        :rtype: object
        """
        try:
            instance = self.ui_extender_cls.model.objects.get(**{
                self.parent_model_field: row_id
            })
        except self.ui_extender_cls.model.DoesNotExist:
            # расширяющая модель еще только создается
            return
        return instance

    def _get_params(self, instance, context):
        u"""Дополнение параметров."""
        return {
            'instance': instance
        }

    def after(self, request, context, response):
        u"""
        Получение истанса модели, которая расширяет и биндинг ее в окно.

        :param request: Request
        :type request: django.http.HttpRequest
        :param context: Context
        :type context: m3.actions.context.DeclarativeActionContext
        """
        if not isinstance(response, ExtUIScriptResult):
            # не окно
            return
        assert self.parent_model_field, (
            'No parent model field defined in listener!')
        assert issubclass(self.ui_extender_cls, BaseEditWinExtender)

        # расширение интерфейса
        extender = self.ui_extender_cls(response.data)

        # получение id родительской модели
        row_id = self._get_id(context)
        if not row_id:
            return

        # получение инстанса расширяющей модели
        instance = self._get_instance(row_id)
        if not instance:
            return

        # биндинг в окно
        extender.bind_from_object(instance)
        # установка параметров
        params = self._get_params(instance, context)
        extender.set_params(params)


class BaseSaveListener(object):

    u"""
    Базовый класс листенеров сохранения доп.данных.

    Обращение к данному листенеру из экшна делается через

    ..code::

        self.handle('post_save', (obj, context))

    после вызова сохранения ``obj`` (в т.ч. внутри транзакции),
    где obj - инстанс родительской модели (``parent_model_instance``)
    """

    # класс-наследник BaseEditWinExtender
    ui_extender_cls = None
    # имя поля, ссылающуюся на родительскую модель
    parent_model_field = None

    def _get_instance(self, parent_model_instance, context):
        u"""
        Получение инстанса расширяющей модели.

        Метод вынесен для возможности инстанцирования дополнительных
        зависимых моделей (относительно расширяющей модели ``instance``)

        :param parent_model_instance: инстанс расширяемой модели
        :param context: Context
        :type context: m3.actions.context.DeclarativeActionContext
        :returns: инстанс расширяющей модели
        """
        try:
            instance = self.ui_extender_cls.model.objects.get(**{
                self.parent_model_field: parent_model_instance
            })
        except self.ui_extender_cls.model.DoesNotExist:
            instance = self.ui_extender_cls.model(**{
                self.parent_model_field: parent_model_instance
            })
        return instance

    def post_save(self, (parent_model_instance, context)):
        u"""
        Точка входа для расширения из экшна self.handle('post_save', *).

        :param parent_model_instance: инстанс расширяемой модели
        :param context: Context
        :type context: m3.actions.context.DeclarativeActionContext
        """
        assert self.parent_model_field, (
            'No parent model field defined in listener!')
        assert issubclass(self.ui_extender_cls, BaseEditWinExtender)

        instance = self._get_instance(parent_model_instance, context)
        self.ui_extender_cls.bind_to_object(instance, context)

        self._save_instance(instance)

    def _save_instance(self, instance):
        u"""
        Сохранение расширяющей модели.

        Вынесено для возможности сохранения связанных/зависимых сущностей
        """
        instance.full_clean()
        instance.save()
